package imt.concours;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.junit.jupiter.api.Test;

import imt.ingredients.Ingredient;
import imt.ingredients.Plat;
import imt.personnes.Chef;
import imt.personnes.Genre;
import imt.personnes.Jury;
import imt.personnes.Specialite;

public class ConcoursTest {
	public static Jury JURY_1 = new Jury(1, "nom1", "prenom1", "06", Genre.HOMME);
	public static Jury JURY_2 = new Jury(2, "nom2", "prenom2", "06", Genre.HOMME);
	public static Jury JURY_3 = new Jury(3, "nom3", "prenom3", "06", Genre.HOMME);

	public static Chef CHEF_1 = new Chef(1, "chef1", "prenomchef1", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(1, "plat1", new Ingredient[] {})));
	public static Chef chef2 = new Chef(2, "chef2", "prenomchef2", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(2, "plat2", new Ingredient[] {})));
	public static Chef chef3 = new Chef(3, "chef3", "prenomchef3", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(3, "plat3", new Ingredient[] {})));
	public static Chef chef4 = new Chef(4, "chef4", "prenomchef4", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(4, "plat4", new Ingredient[] {})));
	public static Chef chef5 = new Chef(5, "chef5", "prenomchef5", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(5, "plat5", new Ingredient[] {})));

	public static Proposition PROPOSITION_1 = new Proposition(new Plat(1, "plat1", new Ingredient[] {}), CHEF_1);
	public static Proposition PROPOSITION_2 = new Proposition(new Plat(2, "plat2", new Ingredient[] {}), chef2);
	public static Proposition PROPOSITION_3 = new Proposition(new Plat(3, "plat3", new Ingredient[] {}), chef3);
	public static Proposition PROPOSITION_4 = new Proposition(new Plat(4, "plat4", new Ingredient[] {}), chef4);
	public static Proposition PROPOSITION_5 = new Proposition(new Plat(5, "plat5", new Ingredient[] {}), chef5);
	public static Set<Proposition> PROPOSITIONS = Set.of(
			PROPOSITION_1,
			PROPOSITION_2,
			PROPOSITION_3,
			PROPOSITION_4,
			PROPOSITION_5);

	@Test
	void concours_trois_jurys_cinq_chefs() {
		var propositions = Set.of(
				PROPOSITION_1,
				PROPOSITION_2,
				PROPOSITION_3,
				PROPOSITION_4);

		assertThrows(IllegalArgumentException.class, () -> new Concours(JURY_1, JURY_2, JURY_3, propositions));
	}

	@Test
	void les_chefs_proposent_un_plat_qu_ils_savent_faire() {
		var propositions = Set.of(
				PROPOSITION_1,
				PROPOSITION_2,
				PROPOSITION_3,
				PROPOSITION_4,
				new Proposition(new Plat(99, "je sais pas faire donc exception", new Ingredient[]{}), chef5));

		assertThrows(IllegalArgumentException.class, () -> new Concours(JURY_1, JURY_2, JURY_3, propositions));
	}

	@Test
	void fin_du_concours() {
		Concours concours = new Concours(JURY_1, JURY_2, JURY_3, PROPOSITIONS);

		assertEquals(false, concours.estTermine());

		concours.noter(PROPOSITION_1, 10);
		concours.noter(PROPOSITION_2, 3);
		concours.noter(PROPOSITION_3, 8);
		concours.noter(PROPOSITION_4, 9);
		concours.noter(PROPOSITION_5, 18);

		assertEquals(true, concours.estTermine());
	}

	@Test
	void classement_actuel() {
		Proposition proposition1 = PROPOSITION_1.withNote(Optional.of(4));
		Proposition proposition2 = PROPOSITION_2.withNote(Optional.of(3));
		Proposition proposition3 = PROPOSITION_3.withNote(Optional.of(2));
		Proposition proposition4 = PROPOSITION_4.withNote(Optional.of(1));
		Proposition proposition5 = PROPOSITION_5;

		Concours concours1 = new Concours(JURY_1, JURY_2, JURY_3, Set.of(
			proposition1,
			proposition2,
			proposition3,
			proposition4,
			proposition5
		));

		List<Proposition> expected = new ArrayList<>();
		expected.add(proposition4);
		expected.add(proposition3);
		expected.add(proposition2);
		expected.add(proposition1);
		expected.add(proposition5);

		assertThat(concours1.classement()).isEqualTo(expected);
	}
}
