package imt.concours;

import static imt.concours.ConcoursTest.CHEF_1;
import static imt.concours.ConcoursTest.JURY_1;
import static imt.concours.ConcoursTest.JURY_2;
import static imt.concours.ConcoursTest.JURY_3;
import static imt.concours.ConcoursTest.PROPOSITIONS;
import static imt.concours.ConcoursTest.PROPOSITION_1;
import static imt.concours.ConcoursTest.PROPOSITION_2;
import static imt.concours.ConcoursTest.PROPOSITION_3;
import static imt.concours.ConcoursTest.PROPOSITION_4;
import static imt.concours.ConcoursTest.PROPOSITION_5;
import static org.assertj.core.api.Assertions.assertThat;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.jupiter.api.Test;

import imt.ingredients.Ingredient;
import imt.ingredients.Plat;
import imt.personnes.Chef;
import imt.personnes.Genre;
import imt.personnes.Specialite;

public class FileConcoursTest {
	public static Concours CONCOURS_1() {
		return new Concours(JURY_1, JURY_2, JURY_3, PROPOSITIONS);
	}

	public static Concours CONCOURS_2() {
		return new Concours(JURY_1, JURY_2, JURY_3, PROPOSITIONS);
	}

	public static Concours CONCOURS_3() {
		return new Concours(JURY_1, JURY_2, JURY_3, PROPOSITIONS);
	}

	@Test
	void fileDeConcours_tout_les_concours_du_chef() {
		Set<Proposition> propositionsSansTelChef = new HashSet<>(PROPOSITIONS);
		propositionsSansTelChef.remove(PROPOSITION_1);
		propositionsSansTelChef.add(
				new Proposition(new Plat(99, "plat99", new Ingredient[] {}), new Chef(99, "nom99", "prenom99", "QSD",
						Genre.HOMME, 4, Specialite.CUISINIER, Set.of(new Plat(99, "plat99", new Ingredient[] {})))));

		Concours concoursSansTelChef = new Concours(JURY_1, JURY_2, JURY_3, propositionsSansTelChef);

		FileConcours fileConcours = new FileConcours(CONCOURS_1(), concoursSansTelChef, CONCOURS_3());

		assertThat(fileConcours.concoursAuquelParticipe(CHEF_1)).isEqualTo(List.of(CONCOURS_1(), CONCOURS_3()));
	}

	@Test
	void concours_termines() {
		Concours CONCOURS_1 = CONCOURS_1();
		FileConcours fileConcours = new FileConcours(CONCOURS_1, CONCOURS_2(), CONCOURS_3());
		assertThat(fileConcours.passerAuConcoursSuivant()).isFalse();
		forcer_la_fin_du_concours(CONCOURS_1);

		assertThat(fileConcours.passerAuConcoursSuivant()).isTrue();

		assertThat(fileConcours.concoursTermines()).isEqualTo(List.of(CONCOURS_1));
	}

	void forcer_la_fin_du_concours(Concours concours) {
		assertThat(concours.estTermine()).isFalse();
		concours.noter(PROPOSITION_1, 10);
		concours.noter(PROPOSITION_2, 20);
		concours.noter(PROPOSITION_3, 30);
		concours.noter(PROPOSITION_4, 40);
		concours.noter(PROPOSITION_5, 50);
		assertThat(concours.estTermine()).isTrue();
	}

	@Test
	void concours_auquel_un_chef_a_participe() {
		Concours CONCOURS_1 = CONCOURS_1();
		Concours CONCOURS_2 = CONCOURS_2();

		FileConcours fileConcours = new FileConcours(CONCOURS_1, CONCOURS_2, CONCOURS_3());
		forcer_la_fin_du_concours(CONCOURS_1);
		fileConcours.passerAuConcoursSuivant();
		forcer_la_fin_du_concours(CONCOURS_2);
		fileConcours.passerAuConcoursSuivant();

		assertThat(fileConcours.concoursAuquelAParticipe(CHEF_1)).isEqualTo(List.of(CONCOURS_1, CONCOURS_2));
	}
}
